import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from pandas import json_normalize
from math import sin, cos, sqrt, atan2, radians

# Universal file saving path
cwd = os.getcwd()

# SINGLE GRAPH
def singleGraph(csv, x_col, y_col, width, height, x_max, y_max, x_label, y_label, graph_title):

  # Pre-set style
  plt.style.use("fivethirtyeight")

  # File prep
  data = pd.read_csv(csv)

  # Metric
  x = data[x_col]
  y = data[y_col]

  # Assign graph settings

  plt.figure(figsize=(width,height))

  plt.xlabel(x_label)
  plt.ylabel(y_label)
  plt.title(graph_title)

  if x_max != '':
    plt.xlim(0,int(x_max))
  else:
    pass
  
  if y_max != '':
    plt.ylim(0,int(y_max))
  else:
    pass

  plt.plot(x,y)

  plt.savefig('/content/single-graph.svg', format="svg")

# MULTI GRAPH
def multiGraph(csv, x_col, y_col_1, y_col_2, width, height, x_label, y_label_1, y_label_2, graph_title):

  line_1 = 'steelblue'
  line_2 = 'red'

  # File prep
  data = pd.read_csv(csv)

  # Metric
  x = data[x_col]
  y1 = data[y_col_1]
  y2 = data[y_col_2]


  fig,ax = plt.subplots(figsize=(width, height))

  ax.plot(x, y1, color=line_1, linewidth=3)
  ax.set_xlabel(x_label, fontsize=14)
  ax.set_ylabel(y_label_1, color=line_1, fontsize=16)

  ax2 = ax.twinx()

  ax2.plot(x, y2, color=line_2, linewidth=2)
  ax2.set_ylabel(y_label_2, color=line_2, fontsize=16)

  plt.savefig('/content/multi-graph.svg', format="svg")


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from pandas import json_normalize
from math import sin, cos, sqrt, atan2, radians

def queryByActivityExactName(name):

  # Query the rows with the name
  result = activities_list.loc[activities_list['name'] == name]

  return result

def queryByActivityApproximateName(word):
 
  # Query the rows with the name
  results = activities_list[activities_list['name'].str.contains(word)]

  return results

# Find the Lat Lon of velocity of interest

def queryBySingleVelocity(velocity):

  search_string = str(velocity)[:3]

  try:
    result = datastream.loc[datastream['velocity_smooth'].astype(str).str[:3] == search_string]
  except ValueError:
    print("Error: Non-numeric values found in 'values' column.")

  return result

# Find the Lat Lon of a range of velocities of interest

def queryByVelocityRange(velocity_min, velocity_max):
  # Minimum velocity

  v1 = str(velocity_min)[:3]

  # Maximum velocity

  v2 = str(velocity_max)[:3]

  try:
    result = datastream.loc[
        (datastream['velocity_smooth'].astype(str).str[:3] >= v1)
        &
        (datastream['velocity_smooth'] .astype(str).str[:3] <= v2)]
  except ValueError:
    print("Error: Non-numeric values found in 'values' column.")

  return result

def queryByRadius(lat, lon, radius):

  # Define a function to calculate the distance between two points using the Haversine formula
  def haversine(lat1, lon1, lat2, lon2):
      R = 6371  # Earth's radius in kilometers
      d_lat = radians(lat2 - lat1)
      d_lon = radians(lon2 - lon1)
      a = sin(d_lat/2)**2 + cos(radians(lat1)) * cos(radians(lat2)) * sin(d_lon/2)**2
      c = 2 * atan2(sqrt(a), sqrt(1-a))
      distance = R * c
      return distance

  # Query the rows where the distance to the center point is less than the radius
  activities_list['distance'] = activities_list.apply(lambda row: haversine(lat, lon, row['latitude'], row['longitude']), axis=1)
  result = activities_list[activities_list['distance'] <= radius]

  # Display the result
  df_filtered = activities_list.loc[activities_list['distance'] <= radius]

  print(df_filtered.loc[:, ['name','id','latitude','longitude']])


import os
import pandas as pd
import numpy as np
from pandas import json_normalize
from math import sin, cos, sqrt, atan2, radians

# Universal file saving path
cwd = os.getcwd()

# ACTIVITIES LIST
def activitiesList (filepath):

  global activities_list

  # File saving preps
  file_name = 'activities-list.csv'
  file_path = os.path.join(cwd,file_name)
  
  data = pd.read_json(filepath)

  # Convert from JSON Pandas Dataframe
  df = pd.DataFrame.from_dict(data)

  # Convert lat-lon to string
  df['start_latlng'] = df['start_latlng'].astype(str)

  # Extract the latitude and longitude values from 'start_latlng' column, then put in separate columns
  df[['latitude', 'longitude']] = df['start_latlng'].str.extract(r'\[(.*),(.*)\]')

  # Convert latitude and longitude data type into float
  df['latitude'] = df['latitude'].astype(float)
  df['longitude'] = df['longitude'].astype(float)

  activities_list = df.drop(['resource_state'], axis=1)

  df.to_csv(file_path) # Save table as CSV with default name "activities-list"

  return activities_list # Display result


# ACTIVITY STREAM
def activityStream(filepath):

  global datastream

  # File saving preps
  file_name = 'activity-stream.csv'
  file_path = os.path.join(cwd,file_name)

  # Read the JSON file
  stream_data_df = pd.read_json(filepath)

  # Convert from JSON to Pandas Dataframe
  a = pd.DataFrame.from_dict(stream_data_df)

  # Explode the arrays into rows
  b = a.apply(pd.Series.explode).reset_index()

  # Convert those fuckers
  b['latlng'] = b['latlng'].astype(str)

  b[['latitude', 'longitude']] = b['latlng'].str.extract(r'\[(.*),(.*)\]')

  # Remove unused columns
  b.drop(b.tail(3).index, inplace=True)
  datastream = b.drop(['latlng','index'], axis=1)

  # Convert the data types
  datastream = datastream.astype({
    'velocity_smooth': 'float',
    'distance': 'float',
    'altitude': 'float',
    'time':'int',
    'latitude': 'float',
    'longitude': 'float', 
    })
  
  datastream.to_csv(file_name)

  return datastream


from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.4'
DESCRIPTION = 'Process and analyse personal geo-data.'
LONG_DESCRIPTION = 'A user-centric Python library for processing and analysing GNSS data, telemetry data, and other spatial-temporal data.'

# Setting up
setup(
    name="Gaiatri",
    version=VERSION,
    author="MotherBunker (A.N.A. Martam)",
    author_email="<motherrrcorp@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['pandas','matplotlib','numpy'],
    keywords=['python','geospatial','geographical information systems','telemetry'],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: End Users/Desktop",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)